@@ -21,14 +21,14 @@ import (
 	"strings"
 	"sync"
 
-	"github.com/casbin/casbin/v2/effector"
-	"github.com/casbin/casbin/v2/log"
-	"github.com/casbin/casbin/v2/model"
-	"github.com/casbin/casbin/v2/persist"
-	fileadapter "github.com/casbin/casbin/v2/persist/file-adapter"
-	"github.com/casbin/casbin/v2/rbac"
-	defaultrolemanager "github.com/casbin/casbin/v2/rbac/default-role-manager"
-	"github.com/casbin/casbin/v2/util"
+	"github.com/casbin/casbin/v3/effector"
+	"github.com/casbin/casbin/v3/log"
+	"github.com/casbin/casbin/v3/model"
+	"github.com/casbin/casbin/v3/persist"
+	fileadapter "github.com/casbin/casbin/v3/persist/file-adapter"
+	"github.com/casbin/casbin/v3/rbac"
+	defaultrolemanager "github.com/casbin/casbin/v3/rbac/default-role-manager"
+	"github.com/casbin/casbin/v3/util"
 
 	"github.com/casbin/govaluate"
 )
@@ -568,6 +568,7 @@ func (e *Enforcer) EnableAcceptJsonRequest(acceptJsonRequest bool) {
 
 // BuildRoleLinks manually rebuild the role inheritance relations.
 func (e *Enforcer) BuildRoleLinks() error {
+	e.invalidateMatcherMap()
 	if e.rmMap == nil {
 		return errors.New("rmMap is nil")
 	}
@@ -658,7 +659,8 @@ func (e *Enforcer) enforce(matcher string, explains *[]string, rvals ...interfac
 	if matcher == "" {
 		expString = e.model["m"][mType].Value
 	} else {
-		expString = util.RemoveComments(util.EscapeAssertion(matcher))
+		// For custom matchers provided at runtime, escape backslashes in string literals
+		expString = util.EscapeStringLiterals(util.RemoveComments(util.EscapeAssertion(matcher)))
 	}
 
 	rTokens := make(map[string]int, len(e.model["r"][rType].Tokens))
