@@ -17,9 +17,9 @@ package casbin
 import (
 	"fmt"
 
-	Err "github.com/casbin/casbin/v2/errors"
-	"github.com/casbin/casbin/v2/model"
-	"github.com/casbin/casbin/v2/persist"
+	Err "github.com/casbin/casbin/v3/errors"
+	"github.com/casbin/casbin/v3/model"
+	"github.com/casbin/casbin/v3/persist"
 )
 
 const (
@@ -34,6 +34,12 @@ func (e *Enforcer) shouldNotify() bool {
 	return e.watcher != nil && e.autoNotifyWatcher
 }
 
+// validateConstraintsForGroupingPolicy validates constraints for grouping policy changes.
+// It returns an error if constraint validation fails.
+func (e *Enforcer) validateConstraintsForGroupingPolicy() error {
+	return e.model.ValidateConstraints()
+}
+
 // addPolicy adds a rule to the current policy.
 func (e *Enforcer) addPolicyWithoutNotify(sec string, ptype string, rule []string) (bool, error) {
 	if e.dispatcher != nil && e.autoNotifyDispatcher {
@@ -63,6 +69,11 @@ func (e *Enforcer) addPolicyWithoutNotify(sec string, ptype string, rule []strin
 		if err != nil {
 			return true, err
 		}
+
+		// Validate constraints after adding grouping policy
+		if err := e.validateConstraintsForGroupingPolicy(); err != nil {
+			return false, err
+		}
 	}
 
 	return true, nil
@@ -106,6 +117,11 @@ func (e *Enforcer) addPoliciesWithoutNotify(sec string, ptype string, rules [][]
 		if err != nil {
 			return true, err
 		}
+
+		// Validate constraints after adding grouping policies
+		if err := e.validateConstraintsForGroupingPolicy(); err != nil {
+			return false, err
+		}
 	}
 
 	return true, nil
@@ -135,6 +151,11 @@ func (e *Enforcer) removePolicyWithoutNotify(sec string, ptype string, rule []st
 		if err != nil {
 			return ruleRemoved, err
 		}
+
+		// Validate constraints after removing grouping policy
+		if err := e.validateConstraintsForGroupingPolicy(); err != nil {
+			return false, err
+		}
 	}
 
 	return ruleRemoved, nil
@@ -166,6 +187,11 @@ func (e *Enforcer) updatePolicyWithoutNotify(sec string, ptype string, oldRule [
 		if err != nil {
 			return ruleUpdated, err
 		}
+
+		// Validate constraints after updating grouping policy
+		if err := e.validateConstraintsForGroupingPolicy(); err != nil {
+			return false, err
+		}
 	}
 
 	return ruleUpdated, nil
@@ -202,6 +228,11 @@ func (e *Enforcer) updatePoliciesWithoutNotify(sec string, ptype string, oldRule
 		if err != nil {
 			return ruleUpdated, err
 		}
+
+		// Validate constraints after updating grouping policies
+		if err := e.validateConstraintsForGroupingPolicy(); err != nil {
+			return false, err
+		}
 	}
 
 	return ruleUpdated, nil
@@ -235,6 +266,11 @@ func (e *Enforcer) removePoliciesWithoutNotify(sec string, ptype string, rules [
 		if err != nil {
 			return rulesRemoved, err
 		}
+
+		// Validate constraints after removing grouping policies
+		if err := e.validateConstraintsForGroupingPolicy(); err != nil {
+			return false, err
+		}
 	}
 	return rulesRemoved, nil
 }
@@ -267,6 +303,11 @@ func (e *Enforcer) removeFilteredPolicyWithoutNotify(sec string, ptype string, f
 		if err != nil {
 			return ruleRemoved, err
 		}
+
+		// Validate constraints after removing filtered grouping policies
+		if err := e.validateConstraintsForGroupingPolicy(); err != nil {
+			return false, err
+		}
 	}
 
 	return ruleRemoved, nil
@@ -322,6 +363,11 @@ func (e *Enforcer) updateFilteredPoliciesWithoutNotify(sec string, ptype string,
 		if err != nil {
 			return oldRules, err
 		}
+
+		// Validate constraints after updating filtered grouping policies
+		if err := e.validateConstraintsForGroupingPolicy(); err != nil {
+			return oldRules, err
+		}
 	}
 
 	return oldRules, nil
