@@ -23,10 +23,10 @@ import (
 	"strconv"
 	"strings"
 
-	"github.com/casbin/casbin/v2/config"
-	"github.com/casbin/casbin/v2/constant"
-	"github.com/casbin/casbin/v2/log"
-	"github.com/casbin/casbin/v2/util"
+	"github.com/casbin/casbin/v3/config"
+	"github.com/casbin/casbin/v3/constant"
+	"github.com/casbin/casbin/v3/log"
+	"github.com/casbin/casbin/v3/util"
 )
 
 // Model represents the whole access control model.
@@ -44,6 +44,7 @@ var sectionNameMap = map[string]string{
 	"g": "role_definition",
 	"e": "policy_effect",
 	"m": "matchers",
+	"c": "constraint_definition",
 }
 
 // Minimal required sections for a model to be valid.
@@ -92,8 +93,13 @@ func (model Model) AddDef(sec string, key string, value string) bool {
 		ast.Value = util.RemoveComments(util.EscapeAssertion(ast.Value))
 	}
 
-	if sec == "m" && strings.Contains(ast.Value, "in") {
-		ast.Value = strings.Replace(strings.Replace(ast.Value, "[", "(", -1), "]", ")", -1)
+	if sec == "m" {
+		// Escape backslashes in string literals to match CSV parsing behavior
+		ast.Value = util.EscapeStringLiterals(ast.Value)
+
+		if strings.Contains(ast.Value, "in") {
+			ast.Value = strings.Replace(strings.Replace(ast.Value, "[", "(", -1), "]", ")", -1)
+		}
 	}
 
 	_, ok := model[sec]
@@ -191,6 +197,11 @@ func (model Model) LoadModelFromText(text string) error {
 	return model.loadModelFromConfig(cfg)
 }
 
+// loadModelFromConfig loads the model from a config interface.
+// It loads all sections defined in sectionNameMap and validates that required sections are present.
+// If constraint_definition section exists, it validates all constraints against the current policy.
+// Note: Constraint validation is performed during model loading, which may affect loading performance
+// and can cause model loading to fail if constraints are violated or invalid.
 func (model Model) loadModelFromConfig(cfg config.ConfigInterface) error {
 	for s := range sectionNameMap {
 		loadSection(model, cfg, s)
@@ -204,6 +215,12 @@ func (model Model) loadModelFromConfig(cfg config.ConfigInterface) error {
 	if len(ms) > 0 {
 		return fmt.Errorf("missing required sections: %s", strings.Join(ms, ","))
 	}
+
+	// Validate constraints after model is loaded
+	if err := model.ValidateConstraints(); err != nil {
+		return err
+	}
+
 	return nil
 }
 
@@ -395,6 +412,12 @@ func (model Model) ToText() string {
 			s.WriteString(fmt.Sprintf("%s = %s\n", ptype, model["g"][ptype].Value))
 		}
 	}
+	if _, ok := model["c"]; ok {
+		s.WriteString("[constraint_definition]\n")
+		for ptype := range model["c"] {
+			s.WriteString(fmt.Sprintf("%s = %s\n", ptype, model["c"][ptype].Value))
+		}
+	}
 	s.WriteString("[policy_effect]\n")
 	writeString("e")
 	s.WriteString("[matchers]\n")
